;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x81f8c82c13e494dfe230db93e73b855d . "./code.cg2js/cg-acache.fasl")
(#xeb6ee4c1945ee037676394970d2d4a7c . "./code.cg2js/cg-base.fasl")
(#x2cab336609bc9105ae17a912db38d2ed . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x5c278d3fbd83726ab1b8b00093911b59 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x961c1ec3851fc92379e5f9024e6dee34 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x5664544bea6d0f3273b2cac306e8cc8e . "./code.cg2js/cg-button.fasl")
(#x63a3ffe6a130e00bc7b5d79ca957440a . "./code.cg2js/cg-calendar.fasl")
(#x137e3d9d43712a0064202d32e998cef2 . "./code.cg2js/cg-caret.fasl")
(#x15fe70a01195e84c2e4e6710c2b22f65 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x1d24d2b899530b9d6698c04dc719e1e3 . "./code.cg2js/cg-chart-widget.fasl")
(#xc4bbba53074155f5fde466f121af2ae2 . "./code.cg2js/cg-check-box.fasl")
(#x6e381aa526ea8410adb3a3eb7f7003ad . "./code.cg2js/cg-choice-list.fasl")
(#x36569b6ab65671f0c090351227205f70 . "./code.cg2js/cg-choose-printer.fasl")
(#x4fed4a9412ee0580993ba3b3461e6cfd . "./code.cg2js/cg-class-grid.fasl")
(#xf2249f75228e07fb4b3e2cdb7c687aaf . "./code.cg2js/cg-class-slot-grid.fasl")
(#xc8f57f22b4ea572d6b8b831a692bd511 . "./code.cg2js/cg-class-support.fasl")
(#xa3593ce136acaff11dc47964efee7424 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xd20cc7c5b63871f4cc3baf2d47f18de0 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x9428bab2631372d41c0d43119a660da0 . "./code.cg2js/cg-clipboard.fasl")
(#x1c58ca4e00c1d76b19e8b8048c1086b8 . "./code.cg2js/cg-color-dialog.fasl")
(#xd9b483a44337565724a6687c35813751 . "./code.cg2js/cg-combo-box.fasl")
(#xf00de8478f9e2b6c96b39f6822bc2951 . "./code.cg2js/cg-common-control.fasl")
(#x20ad11924b4ed2e686269192cabdf420 . "./code.cg2js/cg-comtab.fasl")
(#x4c8308f58b4d9dd3c944a2734eeca875 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x5365ccd14a4ef361517793e5601e912a . "./code.cg2js/cg-curve.fasl")
(#x182f8b3802f88d212636c91bd7a2c9e4 . "./code.cg2js/cg-dde-utils.fasl")
(#x3e69ecf7004e32d32cebdf106fed3c32 . "./code.cg2js/cg-dev.fasl")
(#xa573eb6d085e38938af9806a2ad669ef . "./code.cg2js/cg-dialog-item.fasl")
(#xeb1ad733a4739bf3d2475d377cf3fdf2 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x1f08f913daf82d0de43bdc118ca89e50 . "./code.cg2js/cg-directory-dialog.fasl")
(#x1cbaaf09dfa9ceaaabdb3332191c8ccf . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xc7537deddb332f8e39c87677e21bc683 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x4125bc077ed80b3fcf7dd1b128d5211c . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x2b553e33e906cb26b495d73ae5202686 . "./code.cg2js/cg-drawable.fasl")
(#xff368055f0952f1b26ea5e2bd3cb6e80 . "./code.cg2js/cg-dropping-outline.fasl")
(#xedd501d97ebfcbcaa3d4d3e2d3a9c756 . "./code.cg2js/cg-edit-in-place.fasl")
(#xcd4a7671b45a030e5c8a36c8707804a0 . "./code.cg2js/cg-editable-text.fasl")
(#x21796eed971153c9301da26a0d89e87b . "./code.cg2js/cg-file-dialog.fasl")
(#x6f8fc1a43025999befd390cd02c9ef97 . "./code.cg2js/cg-fill-texture.fasl")
(#xe1ffd2b7f50cf0bdba4124990c28f132 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x74acdf27b6b6dbe145cc9d3957dfcc32 . "./code.cg2js/cg-font-dialog.fasl")
(#x99720994aaa092596a45402d3dad714e . "./code.cg2js/cg-gesture-emulation.fasl")
(#xff467d2a2a2eaf4b7dea5bf436d8c3ef . "./code.cg2js/cg-get-pixmap.fasl")
(#x7d8b0fcc774f2e3c373b6bdd06fbc448 . "./code.cg2js/cg-get-position.fasl")
(#x8ba7b3d61cd1d4f4101c352936419f25 . "./code.cg2js/cg-graphics-context.fasl")
(#x4c314281666707f877a57ad99d685dd1 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x68296045973ba4d8e4cf8c7a00a667ff . "./code.cg2js/cg-grid-widget.fasl")
(#xbfd50620db9251ee8cf13b65b2ec7f2d . "./code.cg2js/cg-group-box.fasl")
(#x5057ce6f0203f497132ce49cd2c93e7c . "./code.cg2js/cg-header-control.fasl")
(#x9c6ba438625bf2e7939bd245b4a55b68 . "./code.cg2js/cg-hotspot.fasl")
(#x81716360e7700b936353b7a19be34b30 . "./code.cg2js/cg-html-dialog.fasl")
(#x47f9da1bb3172c6432a7e2f0c27e552b . "./code.cg2js/cg-html-widget.fasl")
(#x4bcc71e67e4c860e1d5ee12c935df073 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x8c54b41b60cf71a6fa4ca700890959e2 . "./code.cg2js/cg-icon.fasl")
(#x47b25178716fa0dbccac4a0ab910d93d . "./code.cg2js/cg-ie.fasl")
(#x854badd8eeeced1b40478274e2269340 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x6fade0e9f9ea8614099fbddcc9cb680e . "./code.cg2js/cg-intersect.fasl")
(#xbc4b3095ec1d5629dc1831435b1d7b86 . "./code.cg2js/cg-item-list.fasl")
(#x3a9eaf740aba5bcc5f7bacf7dfaa31ce . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xe07966fd5a7ea73559ba3118308b5019 . "./code.cg2js/cg-lamp.fasl")
(#x459cf3fe5ee476f2c6eaae076e0d2b80 . "./code.cg2js/cg-layout.fasl")
(#xd76fdd4a425a8871cd9ea305f0b6c04a . "./code.cg2js/cg-lettered-menu.fasl")
(#xf85a02ba04b24ab05f4f350a3e06ce34 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x846d8c8083f2d26ce5d4166c961c9abd . "./code.cg2js/cg-lisp-text.fasl")
(#x5f83e90748aee171ef77211b5b192257 . "./code.cg2js/cg-lisp-widget.fasl")
(#xcb4fa739d6560b72edcfed95bb69f81a . "./code.cg2js/cg-list-view.fasl")
(#xc84079dc43ca7fbe98799def91395635 . "./code.cg2js/cg-mci.fasl")
(#x4432edb452a8fdbfe1184133106d37cd . "./code.cg2js/cg-menu-tooltip.fasl")
(#x4726f42388b8b2e7cb06855973356556 . "./code.cg2js/cg-menu.fasl")
(#xaf94b187112120bfcfccf0fbba9f0d1e . "./code.cg2js/cg-message-dialog.fasl")
(#xfb4b452204690a6e647b2e0f1022a3d4 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x73de3ad1bbf62cd08040186b6e06c669 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x26977934014832a1722cc25c2a6eb2aa . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xa9933acd58caa941fffadb423a158e50 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x5eafa6fa6cdcc44bec72cfa7ffb73247 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x1dd8e4914b9b3fe02d7f0577e433f013 . "./code.cg2js/cg-nodes.fasl")
(#x9767ad6f322ee8c4e521df0258bcdcf8 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x50c13b0f95a53dedaf70a8ad06a9e841 . "./code.cg2js/cg-object-editor.fasl")
(#xdadc5aa0a67eebace252cb9a4dbb94ac . "./code.cg2js/cg-ocx.fasl")
(#x886d53ad9aa81febedc1a243706e27cd . "./code.cg2js/cg-os-widget.fasl")
(#x95199cc1a09361c73b4911238df60102 . "./code.cg2js/cg-os-window.fasl")
(#x3ea663ea80a24a8ac78494edc733e8b9 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x755fa300a2946eb644e37d2f2a803600 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x1cde3e0a9c7613f5e43cf091fb43e70e . "./code.cg2js/cg-outline.fasl")
(#x2a22fca414d3ae3a38a575acee14ee66 . "./code.cg2js/cg-palette.fasl")
(#x4d08e213da38b140cf3b05f11a847294 . "./code.cg2js/cg-paren-matching.fasl")
(#xd162ac7f6e61adfbe991f3d15b52c554 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xb7db200b860447494fc5021a4a01ba1c . "./code.cg2js/cg-picture-widget.fasl")
(#x87c0a7d884b56dc844b1d62ca2b8d32c . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x8d2a8d9820756eef2dc4a5858172de8f . "./code.cg2js/cg-pixmap-printing.fasl")
(#x4704f0574e529a5c9568289fed4d3881 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xb9bdd4752849fa314e775dcf72fde97b . "./code.cg2js/cg-pixmap-widget.fasl")
(#x60e1f8872d29581f7ec772ad8600edde . "./code.cg2js/cg-pixmap.fasl")
(#x1a276e290d9c827b303ab96cc5fbb4fc . "./code.cg2js/cg-plot-widget.fasl")
(#x07a73b59d5d8639aade4a119ef6e63b5 . "./code.cg2js/cg-printing.fasl")
(#x5f4f71e12b5a21b0a6a45b1c9aa2e9b5 . "./code.cg2js/cg-progress-indicator.fasl")
(#xff8f27bba3e554914bc8954246422e88 . "./code.cg2js/cg-project-window.fasl")
(#x8f4fb912bcc848e5ab7a23ed1cbb2ad7 . "./code.cg2js/cg-property.fasl")
(#xdf384d6d98abfa64d404dc2721ab2424 . "./code.cg2js/cg-radio-button.fasl")
(#xcd745e02782f1cc3585c8cbf8f65372b . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xe5ce250de3f5d5d9ef393975fae5c804 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x75b7c3d10d123c386773d6e95bae8b63 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x35e36c25b0f465e12684a75f71b367a7 . "./code.cg2js/cg-rich-edit.fasl")
(#xc62a52955915404b0798335cac73cbc1 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x5519172bc1a9f6b147842a792d274c99 . "./code.cg2js/cg-scaling-stream.fasl")
(#x2c5bb0769c422465172b33634af03698 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x82cfe75c18d35e1586992b138377f7b4 . "./code.cg2js/cg-scroll-bar.fasl")
(#x486ee65415859f6949e73464f21df794 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x6d7a8b8a0d8380f2d5ea3f622868cb55 . "./code.cg2js/cg-selected-object.fasl")
(#x3eaf8118609462eeacc58cea9ffdde06 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x8d24c908cccae3b97d4e2aadcc32a97a . "./code.cg2js/cg-split-bar.fasl")
(#xdbc5e5deb48cf44f92211324f6d8ac6f . "./code.cg2js/cg-static-text.fasl")
(#x945d1744e2f5cf9326539b23d5655189 . "./code.cg2js/cg-status-bar.fasl")
(#x333ea021ef9b00ccb0d9ad1e05db6028 . "./code.cg2js/cg-string-dialog.fasl")
(#x8f7c8c0dd726b69a48d5011929ab8634 . "./code.cg2js/cg-tab-control.fasl")
(#x99f2fa5d7199dc277d0d24f93e5b750c . "./code.cg2js/cg-template-string.fasl")
(#x928888059f7f9d0b6200605fc0e0eeac . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xa4da16c39bbe9d9f79ba0d5a24f8cc57 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xa098770d7b397aeb9975c4179f11cb94 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xf31d0334793994712d31c34c875cd884 . "./code.cg2js/cg-text-or-combo.fasl")
(#xe5c75556c7db7e11a7900aa2934a8d08 . "./code.cg2js/cg-text-widget.fasl")
(#xecda0f102c447e3e00f1e5b34251d22d . "./code.cg2js/cg-timer.fasl")
(#xebef80b72000013e31cd35bb8b829b0b . "./code.cg2js/cg-toggling-widget.fasl")
(#xe5837cee83abc8f50796667781394368 . "./code.cg2js/cg-toolbar.fasl")
(#xfea19179b6efb1f81aae837d40c396f0 . "./code.cg2js/cg-tooltip.fasl")
(#x9b5778c44b9ba9cc878d4023267f7fb5 . "./code.cg2js/cg-touch.fasl")
(#xdfc00c2d29c6847c4513cacde5cb68c9 . "./code.cg2js/cg-trackbar.fasl")
(#x916795621ead3e028344dc888fb6a552 . "./code.cg2js/cg-tray.fasl")
(#x0160215b86771d75141b34b013e9feb8 . "./code.cg2js/cg-up-down-control.fasl")
(#xea9731daa35617ed22a9393849da9c65 . "./code.cg2js/cg-utility-dialog.fasl")
(#xcfca59633703f9510d93d429abbaa5e7 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x593ae9cc2aa14832952fb2f8d41f3590 . "./code.cg2js/cg-web-browser.fasl")
(#x894f5e7c003412d959bc053ddb607e3b . "./code.cg2js/cg-wrap-string.fasl")
(#x097e66d2bcf3d809073fc25c83cc3897 . "./code.cg2js/cg-yes-no-list.fasl")
(#x733f33aeef137036eb7900f43b6a0633 . "./code.cg2js/cg-yes-no-string.fasl")
(#xea4369c327f8b420ae171d625f55e2b8 . "./code.cg2js/cg.fasl")
(#x335dd06c1f899451b364ef4776398959 . "./code.idejs/ide-acache.fasl")
(#x4bb3b0d8cda07452e2ab3b70b2b69617 . "./code.idejs/ide-apropos.fasl")
(#x122ecc939372454a642d095d8dc43ee1 . "./code.idejs/ide-base.fasl")
(#x0722663da9798549bf6f44ff0fbd0aeb . "./code.idejs/ide-class-browser.fasl")
(#xf41677d3c047aa38f3fd7c3e08a8c8ae . "./code.idejs/ide-class-grapher.fasl")
(#x88886aa7b6be4a4181fad10948235393 . "./code.idejs/ide-clipboard.fasl")
(#xb9ce650a55af1cf25f4b6e43732004a1 . "./code.idejs/ide-dev.fasl")
(#x9d3a9a45a4c1189fc0bc6393097759e3 . "./code.idejs/ide-editor.fasl")
(#xc85e45874af544b848d0bb2c22cb75bc . "./code.idejs/ide-find-definitions.fasl")
(#x409735c6f1f2beafaa720e6f48c24969 . "./code.idejs/ide-find-in-files.fasl")
(#x235e17cf177951d92614f5816eaa607a . "./code.idejs/ide-form.fasl")
(#x08f15ff6c587f93565d701c08cac03d9 . "./code.idejs/ide-grapher.fasl")
(#x24374618be71a4617b4ddd02e5a1699a . "./code.idejs/ide-inspector.fasl")
(#x99a86a0fe3c7249e624ba1acdbd13e44 . "./code.idejs/ide-intro.fasl")
(#xf3774cd6ccbe0a178b713e30ddf449eb . "./code.idejs/ide-keyshort.fasl")
(#x808068f7e8baec391824e8cc52d17706 . "./code.idejs/ide-navigator.fasl")
(#xeeea4fe1c05c8186f2d647b71c899b52 . "./code.idejs/ide-package-list.fasl")
(#x3d33c3d87338978b4b17b65daaf0fb23 . "./code.idejs/ide-patch-dialog.fasl")
(#x66a8becdf2420622b4046f9bbd6af703 . "./code.idejs/ide-process-browser.fasl")
(#xe05948e0bcea3636bdfb6479e07a44f9 . "./code.idejs/ide-project.fasl")
(#x2e04ca5811781db073965e79f15502b5 . "./code.idejs/ide-trace.fasl")
(#xccfa0f93d1f2d76e587a5423030a4048 . "./code.idejs/ide-window-list.fasl")
(#x04276e219ea72b7d91fd462ed0ff5041 . "./code.idejs/ide.fasl")
(#x7f480facc219636edfe865d5e108e9cb . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
